@@ -1,4 +1,5 @@
+from .tools import *  # noqa
 from .viewers import *  # noqa
-from .unified_slider.unified_slider import *  # noqa
 from .parsers import *  # noqa
 from .moment_maps.moment_maps import *  # noqa
+from .slice.slice import * # noqa
