@@ -1,5 +1,6 @@
 import re
 import numpy as np
+from copy import deepcopy
 
 import astropy.units as u
 from specutils import Spectrum1D
@@ -14,6 +15,7 @@ from jdaviz.core.template_mixin import (PluginTemplateMixin,
                                         SpectralSubsetSelectMixin,
                                         SubsetSelect,
                                         DatasetSelectMixin,
+                                        DatasetSpectralSubsetValidMixin,
                                         AutoTextField,
                                         AddResultsMixin)
 from jdaviz.core.custom_traitlets import IntHandleEmpty
@@ -36,7 +38,8 @@ class _EmptyParam:
 
 @tray_registry('g-model-fitting', label="Model Fitting", viewer_requirements='spectrum')
 class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
-                   SpectralSubsetSelectMixin, AddResultsMixin):
+                   SpectralSubsetSelectMixin, DatasetSpectralSubsetValidMixin,
+                   AddResultsMixin):
     """
     See the :ref:`Model Fitting Plugin Documentation <specviz-model-fitting>` for more details.
 
@@ -58,11 +61,17 @@ class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
     * :meth:`model_components`
     * :meth:`get_model_component`
     * :meth:`set_model_component`
+    * :meth:`reestimate_model_parameters`
     * ``equation`` (:class:`~jdaviz.core.template_mixin.AutoTextField`)
-    * ``add_results`` (:class:`~jdaviz.core.template_mixin.AddResults`)
     * ``cube_fit``
       Only exposed for Cubeviz.  Whether to fit the model to the cube instead of to the
       collapsed spectrum.
+    * ``add_results`` (:class:`~jdaviz.core.template_mixin.AddResults`)
+    * ``residuals_calculate`` (bool)
+      Whether to calculate and expose the residuals (model minus data).
+    * ``residuals`` (:class:`~jdaviz.core.template_mixin.AutoTextField`)
+      Label of the residuals to apply when calling :meth:`calculate_fit` if ``residuals_calculate``
+      is ``True``.
     * :meth:`calculate_fit`
     """
     dialog = Bool(False).tag(sync=True)
@@ -93,22 +102,28 @@ class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
 
     cube_fit = Bool(False).tag(sync=True)
 
+    # residuals (non-cube fit only)
+    residuals_calculate = Bool(False).tag(sync=True)
+    residuals_label = Unicode().tag(sync=True)
+    residuals_label_default = Unicode().tag(sync=True)
+    residuals_label_auto = Bool(True).tag(sync=True)
+    residuals_label_invalid_msg = Unicode('').tag(sync=True)
+
     def __init__(self, *args, **kwargs):
-        self._spectrum1d = None
         super().__init__(*args, **kwargs)
 
         self._default_spectrum_viewer_reference_name = kwargs.get(
             "spectrum_viewer_reference_name", "spectrum-viewer"
         )
-
+        self._default_flux_viewer_reference_name = kwargs.get(
+            "flux_viewer_reference_name", "flux-viewer"
+        )
         self._units = {}
         self._fitted_model = None
         self._fitted_spectrum = None
         self.component_models = []
         self._initialized_models = {}
         self._display_order = False
-        self._window = None
-        self._original_mask = None
         if self.config == "cubeviz":
             self.spatial_subset = SubsetSelect(self,
                                                'spatial_subset_items',
@@ -136,6 +151,9 @@ class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
         self.equation = AutoTextField(self, 'model_equation', 'model_equation_default',
                                       'model_equation_auto', 'model_equation_invalid_msg')
 
+        self.residuals = AutoTextField(self, 'residuals_label', 'residuals_label_default',
+                                       'residuals_label_auto', 'residuals_label_invalid_msg')
+
         # set the filter on the viewer options
         self._update_viewer_filters()
 
@@ -146,8 +164,8 @@ class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
             expose += ['spatial_subset']
         expose += ['spectral_subset', 'model_component', 'poly_order', 'model_component_label',
                    'model_components', 'create_model_component', 'remove_model_component',
-                   'get_model_component', 'set_model_component',
-                   'equation', 'add_results']
+                   'get_model_component', 'set_model_component', 'reestimate_model_parameters',
+                   'equation', 'add_results', 'residuals_calculate', 'residuals']
         if self.config == "cubeviz":
             expose += ['cube_fit']
         expose += ['calculate_fit']
@@ -274,6 +292,20 @@ class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
         else:
             return False
 
+    def _get_1d_spectrum(self):
+        # retrieves the 1d spectrum (accounting for spatial subset for cubeviz, if necessary)
+        if self.config == 'cubeviz' and self.spatial_subset_selected != 'Entire Cube':
+            # then we're acting on the auto-collapsed data in the spectrum-viewer
+            # of a spatial subset.  In the future, we may want to expose on-the-fly
+            # collapse options... but right now these will follow the settings of the
+            # spectrum-viewer itself
+            return self.app.get_data_from_viewer(
+                                                 self._default_spectrum_viewer_reference_name,
+                                                 self.spatial_subset_selected
+                                                 )
+        else:
+            return self.dataset.selected_obj
+
     @observe("dataset_selected", "spatial_subset_selected")
     def _dataset_selected_changed(self, event=None):
         """
@@ -293,17 +325,7 @@ class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
             # during initial init, this can trigger before the component is initialized
             return
 
-        if self.config == 'cubeviz' and self.spatial_subset_selected != 'Entire Cube':
-            # then we're acting on the auto-collapsed data in the spectrum-viewer
-            # of a spatial subset.  In the future, we may want to expose on-the-fly
-            # collapse options... but right now these will follow the settings of the
-            # spectrum-viewer itself
-            selected_spec = self.app.get_data_from_viewer(
-                self._default_spectrum_viewer_reference_name,
-                self.spatial_subset_selected
-            )
-        else:
-            selected_spec = self.dataset.selected_obj
+        selected_spec = self._get_1d_spectrum()
         if selected_spec is None:
             return
 
@@ -311,31 +333,11 @@ class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
         # (won't affect calculations because these locations are masked)
         selected_spec.flux[np.isnan(selected_spec.flux)] = 0.0
 
-        # Save original mask so we can reset after applying a subset mask
-        self._original_mask = selected_spec.mask
-
         self._units["x"] = str(
             selected_spec.spectral_axis.unit)
         self._units["y"] = str(
             selected_spec.flux.unit)
 
-        self._spectrum1d = selected_spec
-
-        # Also set the spectral min and max to default to the full range
-        # This is no longer needed for 1D but is preserved for now pending
-        # fixes to Cubeviz for multi-subregion subsets
-        self._window = None
-
-    @observe("spectral_subset_selected")
-    def _on_spectral_subset_selected(self, event):
-        # TODO: does this window not account for gaps?  Should we add the warning?
-        # or can this be removed (see note above in _dataset_selected_changed)
-        if self.spectral_subset_selected == "Entire Spectrum":
-            self._window = None
-        else:
-            spectral_min, spectral_max = self.spectral_subset.selected_min_max(self._spectrum1d)
-            self._window = u.Quantity([spectral_min, spectral_max])
-
     def _default_comp_label(self, model, poly_order=None):
         abbrevs = {'BlackBody': 'BB', 'PowerLaw': 'PL', 'Lorentz1D': 'Lo'}
         abbrev = abbrevs.get(model, model[0].upper())
@@ -415,9 +417,6 @@ class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
             Will raise an error if provided and ``model_component`` is not "Polynomial1D".
             If not provided, will default according to ``poly_order``.
         """
-        if not self._spectrum1d:
-            self._dataset_selected_changed()
-
         model_comp = model_component if model_component is not None else self.model_comp_selected
 
         if model_comp != "Polynomial1D" and poly_order is not None:
@@ -441,6 +440,13 @@ class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
         if comp_label in [cm['id'] for cm in self.component_models]:
             raise ValueError(f"model component label '{comp_label}' already in use")
 
+        new_model = self._initialize_model_component(model_comp, comp_label, poly_order=poly_order)
+        self.component_models = self.component_models + [new_model]
+        # update the default label (likely adding the suffix)
+        self._update_comp_label_default()
+        self._update_model_equation_default()
+
+    def _initialize_model_component(self, model_comp, comp_label, poly_order=None):
         new_model = {"id": comp_label, "model_type": model_comp,
                      "parameters": [], "model_kwargs": {}}
         model_cls = MODELS[model_comp]
@@ -473,12 +479,14 @@ class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
 
             initial_values[param_name] = initial_val
 
+        masked_spectrum = self._apply_subset_masks(self._get_1d_spectrum(), self.spectral_subset)
+        mask = masked_spectrum.mask
         initialized_model = initialize(
             MODELS[model_comp](name=comp_label,
                                **initial_values,
                                **new_model.get("model_kwargs", {})),
-            self._spectrum1d.spectral_axis,
-            self._spectrum1d.flux)
+            masked_spectrum.spectral_axis[~mask] if mask is not None else masked_spectrum.spectral_axis,  # noqa
+            masked_spectrum.flux[~mask] if mask is not None else masked_spectrum.flux)
 
         # need to loop over parameters again as the initializer may have overridden
         # the original default value
@@ -492,10 +500,7 @@ class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
         self._initialized_models[comp_label] = initialized_model
 
         new_model["Initialized"] = True
-        self.component_models = self.component_models + [new_model]
-        # update the default label (likely adding the suffix)
-        self._update_comp_label_default()
-        self._update_model_equation_default()
+        return new_model
 
     def remove_model_component(self, model_component_label):
         """
@@ -587,6 +592,47 @@ class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
 
         return parameter
 
+    def vue_reestimate_model_parameters(self, model_component_label=None, **kwargs):
+        self.reestimate_model_parameters(model_component_label=model_component_label)
+
+    def reestimate_model_parameters(self, model_component_label=None):
+        """
+        Re-estimate all free parameters in a given model component given the currently selected
+        data and subset selections.
+
+        Parameters
+        ----------
+        model_component_label : str or None.
+            The label given to the existing model component.  If None, will iterate over all model
+            components.
+        """
+        if model_component_label is None:
+            return [self.reestimate_model_parameters(model_comp["id"])
+                    for model_comp in self.component_models]
+
+        try:
+            model_index, model_comp = [(i, x) for i, x in enumerate(self.component_models)
+                                       if x["id"] == model_component_label][0]
+        except IndexError:
+            raise ValueError(f"'{model_component_label}' is not a label of an existing model component")  # noqa
+
+        # store user-fixed parameters so we can revert after re-initializing
+        fixed_params = {p['name']: p for p in model_comp['parameters'] if p['fixed']}
+
+        new_model = self._initialize_model_component(model_comp['model_type'],
+                                                     model_component_label,
+                                                     poly_order=model_comp['model_kwargs'].get('degree', None))  # noqa
+
+        # revert fixed parameters to user-value
+        new_model['parameters'] = [fixed_params.get(p['name'], p) for p in new_model['parameters']]
+
+        self.component_models[model_index] = new_model
+        # length hasn't changed, so we need to force the traitlet to update
+        self.send_state("component_models")
+
+        # return user-friendly info on revised model
+        return self.get_model_component(model_component_label)
+
     @property
     def model_components(self):
         """
@@ -618,6 +664,10 @@ class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
         label_comps += ["model"]
         self.results_label_default = " ".join(label_comps)
 
+    @observe("results_label")
+    def _set_residuals_label_default(self, event={}):
+        self.residuals_label_default = self.results_label+" residuals"
+
     @observe("cube_fit")
     def _update_viewer_filters(self, event={}):
         if event.get('new', self.cube_fit):
@@ -641,7 +691,12 @@ class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
         -------
         fitted model
         fitted spectrum/cube
+        residuals (if ``residuals_calculate`` is set to ``True``)
         """
+        if not self.spectral_subset_valid:
+            valid, spec_range, subset_range = self._check_dataset_spectral_subset_valid(return_ranges=True)  # noqa
+            raise ValueError(f"spectral subset '{self.spectral_subset.selected}' {subset_range} is outside data range of '{self.dataset.selected}' {spec_range}")  # noqa
+
         if self.cube_fit:
             return self._fit_model_to_cube(add_data=add_data)
         else:
@@ -660,23 +715,16 @@ class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
             return
         models_to_fit = self._reinitialize_with_fixed()
 
-        # Apply mask from selected subset
-        if self.spectral_subset_selected != "Entire Spectrum":
-            subset_mask = self.app.get_data_from_viewer(
-                self._default_spectrum_viewer_reference_name,
-                data_label=self.spectral_subset_selected
-            ).mask # noqa
-            if self._spectrum1d.mask is None:
-                self._spectrum1d.mask = subset_mask
-            else:
-                self._spectrum1d.mask += subset_mask
+        masked_spectrum = self._apply_subset_masks(self._get_1d_spectrum(), self.spectral_subset)
 
         try:
             fitted_model, fitted_spectrum = fit_model_to_spectrum(
-                self._spectrum1d,
+                masked_spectrum,
                 models_to_fit,
                 self.model_equation,
-                run_fitter=True)
+                run_fitter=True,
+                window=None
+            )
         except AttributeError:
             msg = SnackbarMessage("Unable to fit: model equation may be invalid",
                                   color="error", sender=self)
@@ -688,7 +736,16 @@ class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
 
         if add_data:
             self.app.fitted_models[self.results_label] = fitted_model
-            self._register_spectrum({"spectrum": fitted_spectrum})
+            self.add_results.add_results_from_plugin(fitted_spectrum)
+
+            if self.residuals_calculate:
+                # NOTE: this will NOT load into the viewer since we have already called
+                # add_results_from_plugin above.
+                self.add_results.add_results_from_plugin(masked_spectrum-fitted_spectrum,
+                                                         label=self.residuals.value,
+                                                         replace=False)
+
+        self._set_default_results_label()
 
         # Update component model parameters with fitted values
         if isinstance(self._fitted_model, QuantityModel):
@@ -700,9 +757,8 @@ class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
         # as the starting point for cube fitting
         self._update_initialized_parameters()
 
-        # Reset the data mask in case we use a different subset next time
-        self._spectrum1d.mask = self._original_mask
-
+        if self.residuals_calculate:
+            return fitted_model, fitted_spectrum, masked_spectrum-fitted_spectrum
         return fitted_model, fitted_spectrum
 
     def _fit_model_to_cube(self, add_data):
@@ -726,7 +782,7 @@ class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
             return
 
         # Get the primary data component
-        spec = data.get_object(Spectrum1D, statistic=None)
+        spec = data.get_object(cls=Spectrum1D, statistic=None)
 
         snackbar_message = SnackbarMessage(
             "Fitting model to cube...",
@@ -736,13 +792,18 @@ class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
         # Retrieve copy of the models with proper "fixed" dictionaries
         models_to_fit = self._reinitialize_with_fixed()
 
+        # Apply masks from selected subsets
+        for subset in [self.spatial_subset, self.spectral_subset]:
+            spec = self._apply_subset_masks(spec, subset)
+
         try:
             fitted_model, fitted_spectrum = fit_model_to_spectrum(
                 spec,
                 models_to_fit,
                 self.model_equation,
                 run_fitter=True,
-                window=self._window)
+                window=None
+            )
         except ValueError:
             snackbar_message = SnackbarMessage(
                 "Cube fitting failed",
@@ -779,25 +840,39 @@ class ModelFitting(PluginTemplateMixin, DatasetSelectMixin,
 
         return fitted_model, output_cube
 
-    def _register_spectrum(self, event):
+    def _apply_subset_masks(self, spectrum, subset_component):
         """
-        Add a spectrum to the data collection based on the currently displayed
-        parameters (these could be user input or fit values).
+        For a spectrum/spectral cube ``spectrum``, add a mask attribute
+        if none exists. Mask excludes non-selected spectral and/or
+        spatial subsets.
         """
-        if self._warn_if_no_equation():
-            return
-        # Make sure the initialized models are updated with any user-specified
-        # parameters
-        self._update_initialized_parameters()
+        # only look for a mask if there is a selected subset:
+        if subset_component.selected == subset_component.default_text:
+            return spectrum
 
-        # Need to run the model fitter with run_fitter=False to get spectrum
-        if "spectrum" in event:
-            spectrum = event["spectrum"]
+        spectrum = deepcopy(spectrum)
+        subset_mask = subset_component.selected_subset_mask
+
+        if spectrum.mask is not None:
+            if subset_mask.ndim == 3:
+                if spectrum.mask.ndim == 1:
+                    # if subset mask is 3D and the `spectrum` mask is 1D, which
+                    # happens when `spectrum` has been collapsed from 3D->1D,
+                    # then also collapse the 3D mask in the spatial
+                    # dimensions (0, 1) so that slices in the spectral axis that
+                    # are masked in all pixels become masked in the spectral subset:
+                    subset_mask = np.all(subset_mask, axis=(0, 1))
+            spectrum.mask |= subset_mask
         else:
-            model, spectrum = fit_model_to_spectrum(self._spectrum1d,
-                                                    self._initialized_models.values(),
-                                                    self.model_equation,
-                                                    window=self._window)
+            if subset_mask.ndim < spectrum.flux.ndim:
+                # correct the shape of spectral/spatial axes when they're different:
+                subset_mask = np.broadcast_to(subset_mask, spectrum.flux.shape)
 
-        self.add_results.add_results_from_plugin(spectrum)
-        self._set_default_results_label()
+            elif (subset_mask.ndim == spectrum.flux.ndim and
+                  subset_mask.shape != spectrum.flux.shape):
+                # if the number of dimensions is correct but shape is
+                # different, rearrange the arrays for specutils:
+                subset_mask = np.swapaxes(subset_mask, 1, 0)
+
+            spectrum.mask = subset_mask
+        return spectrum
