@@ -22,6 +22,13 @@ class BaseLinkHandler:
         self.imviz.link_data(link_type='pixels', error_on_fail=True)
         self.check_all_pixel_links()
 
+    @property
+    def default_viewer_limits(self):
+        return (self.imviz.default_viewer.state.x_min,
+                self.imviz.default_viewer.state.x_max,
+                self.imviz.default_viewer.state.y_min,
+                self.imviz.default_viewer.state.y_max)
+
 
 class TestLink_WCS_NoWCS(BaseImviz_WCS_NoWCS, BaseLinkHandler):
 
@@ -88,6 +95,9 @@ class TestLink_WCS_FakeWCS(BaseImviz_WCS_NoWCS, BaseLinkHandler):
 class TestLink_WCS_WCS(BaseImviz_WCS_WCS, BaseLinkHandler):
 
     def test_wcslink_affine_with_extras(self):
+        orig_pixel_limits = self.default_viewer_limits
+        assert_allclose(orig_pixel_limits, (-0.5, 9.5, -0.5, 9.5))
+
         self.imviz.link_data(link_type='wcs', wcs_fallback_scheme=None, error_on_fail=True)
         links = self.imviz.app.data_collection.external_links
         assert len(links) == 1
@@ -95,6 +105,11 @@ class TestLink_WCS_WCS(BaseImviz_WCS_WCS, BaseLinkHandler):
 
         assert self.viewer.get_link_type('has_wcs_2[SCI,1]') == 'wcs'
 
+        # linking should not change axes limits, but should when resetting
+        assert_allclose(self.default_viewer_limits, orig_pixel_limits)
+        self.imviz.default_viewer.state.reset_limits()
+        assert_allclose(self.default_viewer_limits, (-1.5, 9.5, -1, 10))
+
         # Customize display on second image (last loaded).
         self.viewer.set_colormap('Viridis')
         self.viewer.stretch = 'sqrt'
@@ -111,7 +126,8 @@ class TestLink_WCS_WCS(BaseImviz_WCS_WCS, BaseLinkHandler):
         self.viewer.add_markers(tbl, marker_name='xy_markers')
         assert 'xy_markers' in self.imviz.app.data_collection.labels
 
-        # Run linking again, does not matter what kind.
+        # Run linking again with the same options as before (otherwise would fail with an error
+        # since markers now exist)
         self.imviz.link_data(link_type='wcs', wcs_fallback_scheme=None, error_on_fail=True)
 
         # Ensure display is still customized.
@@ -126,10 +142,10 @@ class TestLink_WCS_WCS(BaseImviz_WCS_WCS, BaseLinkHandler):
         assert 'MaskedSubset 1' in all_labels
         assert 'MaskedSubset 2' in all_labels
 
-        # Ensure markers are deleted.
+        # Markers should still exist since the type has not changed
         # Zoom and pan will reset in this case, so we do not check those.
-        assert 'xy_markers' not in self.imviz.app.data_collection.labels
-        assert len(self.viewer._marktags) == 0
+        assert 'xy_markers' in self.imviz.app.data_collection.labels
+        assert len(self.viewer._marktags) == 1
 
         # Pan/zoom.
         self.viewer.center_on((5, 5))
@@ -161,6 +177,15 @@ class TestLink_WCS_WCS(BaseImviz_WCS_WCS, BaseLinkHandler):
         assert self.viewer.label_mouseover.world_ra_deg == '337.5202808000'
         assert self.viewer.label_mouseover.world_dec_deg == '-20.8333330600'
 
+        # Changing link type will raise an error
+        with pytest.raises(ValueError, match="cannot change link_type"):
+            self.imviz.link_data(link_type='pixels', wcs_fallback_scheme=None, error_on_fail=True)
+
+        self.viewer.reset_markers()
+        self.imviz.link_data(link_type='pixels', wcs_fallback_scheme=None, error_on_fail=True)
+        assert 'xy_markers' not in self.imviz.app.data_collection.labels
+        assert len(self.viewer._marktags) == 0
+
     def test_wcslink_fullblown(self):
         self.imviz.link_data(link_type='wcs', wcs_fallback_scheme=None, wcs_use_affine=False,
                              error_on_fail=True)
