@@ -11,7 +11,10 @@ GWCS_LT_0_18_1 = Version(gwcs.__version__) < Version('0.18.1')
 @pytest.mark.remote_data
 @pytest.mark.filterwarnings('ignore')
 def test_plugin(specviz2d_helper):
-    fn = download_file('https://stsci.box.com/shared/static/exnkul627fcuhy5akf2gswytud5tazmw.fits', cache=True)  # noqa
+    # TODO: Change back to smaller number (30?) when ITSD is convinced it is them and not us.
+    #       Help desk ticket INC0183598, J. Quick.
+    fn = download_file('https://stsci.box.com/shared/static/exnkul627fcuhy5akf2gswytud5tazmw.fits',
+                       cache=True, timeout=100)
 
     specviz2d_helper.load_data(spectrum_2d=fn)
 
@@ -42,11 +45,11 @@ def test_plugin(specviz2d_helper):
     trace = pext.export_trace(add_data=True)  # overwrite
     assert isinstance(trace, tracing.FlatTrace)
 
-    # create KosmosTrace
+    # create FitTrace
     pext.trace_trace_selected = 'New Trace'
-    pext.trace_type_selected = 'Auto'
+    pext.trace_type_selected = 'Polynomial'
     trace = pext.export_trace(add_data=True)
-    assert isinstance(trace, tracing.KosmosTrace)
+    assert isinstance(trace, tracing.FitTrace)
     assert trace.guess == 27
     trace = pext.export_trace(trace_pixel=26, add_data=False)
     assert trace.guess == 26
@@ -132,7 +135,7 @@ def test_plugin(specviz2d_helper):
         pext.update_marks(step)
 
     # test exception handling
-    pext.trace_type = 'Auto'
+    pext.trace_type = 'Polynomial'
     pext.bg_type_selected = 'TwoSided'
     pext.bg_separation = 1
     pext.bg_width = 5
