@@ -26,6 +26,7 @@ from jdaviz.core.user_api import UserApiWrapper, PluginUserApi
 __all__ = ['show_widget', 'TemplateMixin', 'PluginTemplateMixin',
            'BasePluginComponent', 'SelectPluginComponent',
            'SubsetSelect', 'SpatialSubsetSelectMixin', 'SpectralSubsetSelectMixin',
+           'DatasetSpectralSubsetValidMixin',
            'ViewerSelect', 'ViewerSelectMixin',
            'LayerSelect', 'LayerSelectMixin',
            'DatasetSelect', 'DatasetSelectMixin',
@@ -944,6 +945,33 @@ class SubsetSelect(SelectPluginComponent):
                         s.label == self.selected][0]
         return subset_group.subset_state
 
+    @property
+    def selected_subset_mask(self):
+        if self._allowed_type == 'spatial':
+            viewer_ref = getattr(self.plugin,
+                                 '_default_flux_viewer_reference_name',
+                                 self.viewers[0].reference_id)
+        elif self._allowed_type == 'spectral':
+            viewer_ref = getattr(self.plugin,
+                                 '_default_spectrum_viewer_reference_name',
+                                 self.viewers[0].reference_id)
+
+        subset = self.app.get_data_from_viewer(
+            viewer_ref, data_label=self.selected
+        )
+
+        if hasattr(subset, 'mask'):
+            # the mask attr is available for spectral subsets:
+            subset_mask = subset.mask
+        else:
+            # `subset` is a GroupedSubset.
+            # We take the logical-not of the mask here, since the glue subset masks
+            # are True in selected regions and False outside. This is the opposite of
+            # the numpy/astropy/specutils convention where True is masked-out.
+            subset_mask = ~subset.to_mask()
+
+        return subset_mask
+
     def selected_min_max(self, spectrum1d):
         if self.selected_obj is None:
             return np.nanmin(spectrum1d.spectral_axis), np.nanmax(spectrum1d.spectral_axis)
@@ -1030,6 +1058,35 @@ class SpatialSubsetSelectMixin(VuetifyTemplate, HubListener):
                                            allowed_type='spatial')
 
 
+class DatasetSpectralSubsetValidMixin(VuetifyTemplate, HubListener):
+    """
+    Adds a traitlet tracking whether self.dataset and self.spectral_subset
+    overlap in the spectral axis.
+
+    Note that if using in another method that is also observing dataset_selected
+    or spectral_subset_selected, that that method could be called before the traitlet
+    is updated.  In that case, call self._check_dataset_spectral_subset_valid() itself
+    directly.  The traitlet can still be used for any warning text in the plugin UI.
+    """
+    spectral_subset_valid = Bool(True).tag(sync=True)
+
+    @observe("dataset_selected", "spectral_subset_selected")
+    def _check_dataset_spectral_subset_valid(self, event={}, return_ranges=False):
+        if self.spectral_subset_selected == "Entire Spectrum":
+            self.spectral_subset_valid = True
+        else:
+            spec = self.dataset.selected_obj
+            spec_min, spec_max = np.nanmin(spec.spectral_axis), np.nanmax(spec.spectral_axis)
+            subset_min, subset_max = self.spectral_subset.selected_min_max(spec)
+            self.spectral_subset_valid = bool(subset_min < spec_max and subset_max > spec_min)
+        if return_ranges:
+            return (self.spectral_subset_valid,
+                    (spec_min.value, spec_max.value),
+                    (subset_min.value, subset_max.value))
+        else:
+            return self.spectral_subset_valid
+
+
 class ViewerSelect(SelectPluginComponent):
     """
     Plugin select for viewers, with support for single or multi-selection.
@@ -1669,7 +1726,7 @@ class AddResults(BasePluginComponent):
         self.label_invalid_msg = ''
         self.label_overwrite = False
 
-    def add_results_from_plugin(self, data_item, replace=None):
+    def add_results_from_plugin(self, data_item, replace=None, label=None):
         """
         Add ``data_item`` to the app's data_collection according to the default or user-provided
         label and adds to any requested viewers.
@@ -1677,6 +1734,9 @@ class AddResults(BasePluginComponent):
         if self.label_invalid_msg:
             raise ValueError(self.label_invalid_msg)
 
+        if label is None:
+            label = self.label
+
         if replace is None:
             replace = self.viewer.selected_reference != 'spectrum-viewer'
 
@@ -1687,29 +1747,29 @@ class AddResults(BasePluginComponent):
             viewer_item = self.app._viewer_item_by_reference(viewer_reference)
             viewer = self.app.get_viewer(viewer_reference)
             viewer_loaded_labels = [layer.layer.label for layer in viewer.layers]
-            add_to_viewer_selected = viewer_reference if self.label in viewer_loaded_labels else 'None'  # noqa
-            visible = self.label in viewer_item['visible_layers']
+            add_to_viewer_selected = viewer_reference if label in viewer_loaded_labels else 'None'  # noqa
+            visible = label in viewer_item['visible_layers']
         else:
             add_to_viewer_selected = self.add_to_viewer_selected
             visible = True
 
-        if self.label in self.app.data_collection:
+        if label in self.app.data_collection:
             if add_to_viewer_selected != 'None':
-                self.app.remove_data_from_viewer(self.viewer.selected_reference, self.label)
-            self.app.data_collection.remove(self.app.data_collection[self.label])
+                self.app.remove_data_from_viewer(self.viewer.selected_reference, label)
+            self.app.data_collection.remove(self.app.data_collection[label])
 
         if not hasattr(data_item, 'meta'):
             data_item.meta = {}
         data_item.meta['Plugin'] = self._plugin.__class__.__name__
         if self.app.config == 'mosviz':
             data_item.meta['mosviz_row'] = self.app.state.settings['mosviz_row']
-        self.app.add_data(data_item, self.label)
+        self.app.add_data(data_item, label)
 
         if add_to_viewer_selected != 'None':
             # replace the contents in the selected viewer with the results from this plugin
             # TODO: switch to an instance/classname check?
             self.app.add_data_to_viewer(self.viewer.selected_id,
-                                        self.label,
+                                        label,
                                         visible=visible, clear_other_data=replace)
 
         # update overwrite warnings, etc
@@ -1776,10 +1836,11 @@ class PlotOptionsSyncState(BasePluginComponent):
     * :meth:`unmix_state`
     """
     def __init__(self, plugin, viewer_select, layer_select, glue_name,
-                 value, sync, state_filter=None):
+                 value, sync, spinner=None, state_filter=None):
         super().__init__(plugin, value=value, sync=sync)
         self._state_filter = state_filter
         self._linked_states = []
+        self._spinner = spinner
         self._processing_change_from_glue = False
         self._processing_change_to_glue = False
         self._cached_properties = ["subscribed_states", "subscribed_icons"]
@@ -1977,6 +2038,9 @@ class PlotOptionsSyncState(BasePluginComponent):
         if self._processing_change_from_glue:
             return
 
+        if self._spinner is not None:
+            setattr(self.plugin, self._spinner, True)
+
         self._processing_change_to_glue = True
         for glue_state in self.linked_states:
             if self._glue_name == 'cmap':
@@ -1991,12 +2055,16 @@ class PlotOptionsSyncState(BasePluginComponent):
 
             if self._glue_name in ['bitmap_visible', 'contour_visible'] and msg['new'] is True:
                 # ensure that the layer is also visible
-                setattr(glue_state, 'visible', msg['new'])
+                if not glue_state.visible:
+                    setattr(glue_state, 'visible', msg['new'])
 
         # need to recompute mixed state
         self._update_mixed_state()
         self._processing_change_to_glue = False
 
+        if self._spinner is not None:
+            setattr(self.plugin, self._spinner, False)
+
     def _on_glue_layer_visible_changed(self, value):
         # this is only triggered for glue_name contour_visible or bitmap_visible
         self._processing_change_from_glue = True
